#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame as pg
# from pysistence import make_list as cria_lista
# from pysistence.persistent_list import PList, EmptyList

from htdp_pt_br.list import *

pg.init()

Fonte = pg.font.SysFont
Cor = pg.color.Color

LARGURA_PADRAO = 400
ALTURA_PADRAO = 300
COR_BRANCO = (255, 255, 255)
cor_fundo = COR_BRANCO

def criar_tela_base(largura, altura, fundo=Cor("white")):
    '''
    Cria tela base do aplicativo.
    :param largura: Int
    :param altura: Int
    :param fundo: Cor
    :return: Tela
    '''
    tela = pg.display.set_mode((largura, altura))
    tela.fill(fundo)
    global cor_fundo
    cor_fundo = fundo
    return tela

tela = criar_tela_base(LARGURA_PADRAO, ALTURA_PADRAO)

def definir_estrutura(nome, campos, mutavel=False):
    '''
    Define uma estrutura de dados compostos (uma classe).
    Por padrão, a estrutura é imutável, mas pode-se criar mutáveis setando 'mutavel=True'.
    :param nome: String -- nome do tipo
    :param campos: List<String> -- lista com os nomes dos campos
    :param mutavel: Boolean [=False]
    :return: retorna um tipo de dado definido pelo programador.
    '''
    if isinstance(campos, tuple):
        campos = list(campos)
    if not mutavel:
        from collections import namedtuple
        return namedtuple(nome, campos)
    else:
        from namedlist import namedlist
        return namedlist(nome, campos)





# VAZIA = cria_lista()
#
#
# def vazia(lista):
#     return not lista
#
# def cons(item, lista):
#     '''
#     Immutable list creator.
#     :param item: Object
#     :param list: Tuple[Object]
#     :return:
#     '''
#     if not isinstance(lista, PList) and not isinstance(lista, EmptyList):
#         raise ValueError("ERRO: Você passou algo que não é uma lista.")
#     return lista.cons(item)
#
#
# def primeiro(lista):
#     '''
#
#     :param list:
#     :return:
#     '''
#     if vazia(lista) or not isinstance(lista, PList):
#         raise ValueError("ERRO: Parece que você tentou pegar o primeiro de uma lista vazia")
#     return lista.first
#
# def resto(lista):
#     '''
#
#     :param list:
#     :return:
#     '''
#     if vazia(lista) or not isinstance(lista, PList):
#         raise ValueError("ERRO: Parece que você tentou pegar o resto de uma lista vazia")
#     return lista.rest
#
#
# def reverte(lista):
#     if vazia(lista):
#         return VAZIA
#     elif not isinstance(lista, PList):
#         raise ValueError("ERRO: Parece que você passou algo que não é uma lista")
#     else:
#         return lista.reverse()
#
#
# def remove(itens, lista):
#     if vazia(lista):
#         return VAZIA
#     elif not isinstance(lista, PList):
#         raise ValueError("ERRO: Parece que você passou algo que não é uma lista")
#     else:
#         return lista.without(itens)
#
# def concatena(lista1, lista2):



###Based on https://github.com/zhemao/funktown


class ListaImutavel(object):
    def __init__(self, *args):
        if len(args) == 0:
            self._vazia = True
            self._cabeca = None
            self._cauda = None

        elif len(args) == 1 and isinstance(args[0], (list, tuple)):
            if len(args[0]) == 0:
                self._vazia = True
                self._cabeca = None
                self._cauda = None
            else:
                self._cabeca = args[0][0]
                if len(args[0]) > 1:
                    self._cauda = ListaImutavel(args[0][1:])
                else: self._cauda = ListaImutavel()
                self._vazia = False

        elif len(args) == 2 and isinstance(args[1], ListaImutavel):
            self._cabeca = args[0]
            self._cauda = args[1]
            self._vazia = False

    def conj(self, itm):
        return ListaImutavel(itm, self)

    @property
    def primeiro(self):
        return self._cabeca

    @property
    def resto(self):
        if self._vazia:
            return self
        return self._cauda

    @property
    def segundo(self):
        if self._vazia:
            return None
        return self._cauda._head

    @property
    def vazia(self):
        return self._vazia


    def reverte(self):
        new_list = ListaImutavel()
        for item in self:
            new_list = new_list.conj(item)
        return new_list

    def concat(self, lista2):
        new_list = lista2
        for item in self.reverte():
            new_list = new_list.conj(item)
        return new_list

    def __contains__(self, itm):
        if self._vazia:
            return False
        if self._cabeca == itm:
            return True
        return itm in self._cauda

    def __iter__(self):
        node = self
        while not node._vazia:
            yield node._cabeca
            node = node._cauda

    def __eq__(self, other):
        if other is None:
            return False

        if not hasattr(other, '__iter__'):
            return False

        node = self

        for itm in other:
            if node._vazia:
                return False
            if node._cabeca != itm:
                return False
            node = node._cauda

        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def __len__(self):
        if self._vazia:
            return 0
        else:
            return 1 + len(self._cauda)

    def __str__(self):
        return '[' + ', '.join([str(x) for x in self]) + ']'

    def __repr__(self):
        return 'ImmutableList(' + str(self) + ')'



def criar_lista(*args):
    return ListaImutavel(args)

VAZIA = criar_lista()

def vazia(lista):
    if not isinstance(lista, ListaImutavel):
        raise ValueError("ERRO: Você passou algo que não é uma lista.")
    return lista.vazia

def conj(item, lista):
    '''
    Immutable list creator.
    :param item: Object
    :param list: Tuple[Object]
    :return:
    '''
    if not isinstance(lista, ListaImutavel):
        raise ValueError("ERRO: Você passou algo que não é uma lista.")
    return lista.conj(item)


def primeiro(lista):
    '''

    :param list:
    :return:
    '''
    if not isinstance(lista, ListaImutavel):
        raise ValueError("ERRO: Você passou algo que não é uma lista.")
    return lista.primeiro

def resto(lista):
    '''

    :param list:
    :return:
    '''
    if not isinstance(lista, ListaImutavel):
        raise ValueError("ERRO: Você passou algo que não é uma lista.")
    return lista.resto


def reverte(lista):
    if not isinstance(lista, ListaImutavel):
        raise ValueError("ERRO: Você passou algo que não é uma lista.")
    else:
        return lista.reverte()


def concat(lista1, lista2):
    if not isinstance(lista1, ListaImutavel) or not isinstance(lista2, ListaImutavel) :
        raise ValueError("ERRO: Você passou algo que não é uma lista.")
    return lista1.concat(lista2)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from htdp_pt_br.base import *
import sys


'''
String, Fonte, Cor, Int -> Surface
'''
def texto(str, fonte, cor, largura):
    """
    Cria uma imagem com base em um texto.
    :param str: String -- o texto.
    :param fonte: Fonte -- a fonte (classe Fonte) do texto.
    :param cor: Cor -- a cor (classe Cor) do texto.
    :param largura: Int -- a largura na qual o texto deve estar contido. 
    """
    def embrulha_texto(texto, fonte, largura):
        
        linhas = texto.replace('\t', '    ').split('\n')
        if largura is None or largura == 0:
            return linhas

        linhas_embrulhadas = []
        for linha in linhas:
            linha = linha.rstrip() + ' '
            if linha == ' ':
                linhas_embrulhadas.append(linha)
                continue

            # Pega o espaço mais à esquerda, ignorando o espaço em branco inicial
            inicio = len(linha) - len(linha.lstrip())
            inicio = linha.index(' ', inicio)
            while inicio + 1 < len(linha):
                # Get the next potential splitting point
                proximo = linha.index(' ', inicio + 1)
                if fonte.size(linha[:proximo])[0] <= largura:
                    inicio = proximo
                else:
                    linhas_embrulhadas.append(linha[:inicio])
                    linha = linha[inicio + 1:]
                    inicio = linha.index(' ')
            linha = linha[:-1]
            if linha:
                linhas_embrulhadas.append(linha)
        return linhas_embrulhadas

    def desenha_lista_de_textos(linhas, fonte, cor=(255, 255, 255)):
        desenhado = [fonte.render(line, True, cor).convert_alpha()
                    for line in linhas]

        altura_linha = fonte.get_linesize()
        largura = max(linha.get_width() for linha in desenhado)
        topos = [int(round(i * altura_linha)) for i in range(len(desenhado))]
        altura = topos[-1] + fonte.get_height()

        surface = pg.Surface((largura, altura)).convert_alpha()
        surface.fill((0, 0, 0, 0))
        for y, linha in zip(topos, desenhado):
            surface.blit(linha, (0, y))

        return surface

    linhas = embrulha_texto(str, fonte, largura)
    return desenha_lista_de_textos(linhas, fonte, cor)



'''
FUNÇÕES PARA CRIAÇÃO DE IMAGENS E FIGURAS GEOMÉTRICAS
'''

'''
Int, Int, Cor -> Imagem
'''
def elipse(largura, altura, cor):
    '''
    Cria imagem de uma elipse.
    :param largura: Int
    :param altura: Int
    :param cor: Cor
    :return: Imagem
    '''
    img = pg.Surface((largura, altura), pg.SRCALPHA)  # imagem vazia
    pg.draw.ellipse(img, cor, (0, 0, largura, altura))
    return img

'''
Int, Int, Cor -> Imagem
'''
def retangulo(largura, altura, cor):
    '''
    Cria imagem de um retângulo.
    :param largura: Int
    :param altura: Int
    :param cor: Cor
    :return: Imagem
    '''
    img = pg.Surface((largura, altura), pg.SRCALPHA)  # imagem vazia
    pg.draw.rect(img, cor, (0, 0, largura, altura))
    return img

'''
Int, Cor -> Imagem
'''
def circulo(raio, cor):
    '''
    Cria imagem de um círculo.
    :param raio: Int
    :param cor: Cor
    :return: Imagem
    '''
    img = pg.Surface((raio*2, raio*2), pg.SRCALPHA)  # imagem vazia
    pg.draw.circle(img, cor, (raio, raio), raio)
    return img

'''
Int, Cor -> Imagem
'''
def quadrado(lado, cor):
    '''
    Cria imagem de um quadrado.
    :param lado: Int
    :param cor: Cor
    :return: Imagem
    '''
    return retangulo(lado, lado, cor)


def poligono(lista_de_pontos, cor, contorno = 0):
    '''
    Cria um polígono com base em uma lista de pontos (considere o canto inferior esquerdo como o ponto (0,0)).
    :param lista_de_pontos: List<Tuple(Int, Int)> -- lista de pares ordenados indicando os pontos que formam o polígono
    :param cor: Cor
    :param contorno [=0]: Int --- use um valor maior que 0 para que o poligono seja contornado. Caso seja 0 (padrão), o polígono sera preenchido.
    :return: Imagem
    '''
    max_x = max_y = 0
    min_x = min_y = sys.maxsize
    for x,y in lista_de_pontos:
        if x > max_x:
            max_x = x
        if y > max_y:
            max_y = y
        if x < min_x:
            min_x = x
        if y < min_y:
            min_y = y
    if contorno > 0:
        for k in range(len(lista_de_pontos)):
            ponto = lista_de_pontos[k]
            if ponto[0] == min_x:
                lista_de_pontos[k] = (ponto[0] + contorno // 2, ponto[1])
            ponto = lista_de_pontos[k]
            if ponto[1] == min_y:
                lista_de_pontos[k] = (ponto[0], ponto[1] + contorno // 2)
    img = pg.Surface((max_x + contorno, max_y + contorno), pg.SRCALPHA)  # imagem vazia
    pg.draw.polygon(img, cor, lista_de_pontos, contorno)
    return img

def arco(largura, altura, angulo_inicio, angulo_fim, cor, contorno=1):
    '''
    Desenha um arco dentro de um espaço definido por largura e altura, dado um angulo inicial e um final.
    :param largura: Int
    :param altura: Int
    :param angulo_inicio: Int -- em graus
    :param angulo_fim: Int -- em graus
    :param contorno: Int
    :return: Imagem
    '''
    import math
    img = pg.Surface((largura, altura), pg.SRCALPHA)  # imagem vazia
    pg.draw.arc(img, cor, (0, 0, largura, altura), math.radians(angulo_inicio), math.radians(angulo_fim), contorno)
    return img


def linha(ponto_inicial, ponto_final, cor, contorno=1):
    '''
    Desenha uma linha, dado um ponto inicial e um ponto final (considere o canto inferior esquerdo como o ponto (0,0)).
    :param ponto_inicial: Tuple(Int, Int)
    :param ponto_final: Tuple(Int, Int)
    :param cor: Cor
    :param contorno: Int
    :return: Imagem
    '''
    max_x = max([ponto_inicial[0], ponto_final[0]])
    max_y = max([ponto_inicial[1], ponto_final[1]])
    img = pg.Surface((max_x, max_y), pg.SRCALPHA)  # imagem vazia
    pg.draw.line(img, cor, ponto_inicial, ponto_final, contorno)
    return img


'''
Int, Int -> Imagem
'''
def folha_transparente(largura, altura):
    '''
    Cria um fundo transparente no qual podem ser colocadas outras imagens.
    :param largura: Int
    :param altura: Int
    :return: Imagem
    '''
    folha = pg.Surface((largura, altura), pg.SRCALPHA)
    return folha

'''
Imagem, Int, Int -> Imagem
'''
def definir_dimensoes(imagem, largura, altura):
    '''
    Define dimensões (largura e altura) de uma imagem.
    :param imagem: Imagem
    :param largura: Int
    :param altura: Int
    :return: Imagem
    '''
    return pg.transform.scale(imagem, (largura, altura))

'''
Imagem, Int -> Imagem
'''
def girar(imagem, angulo):
    '''
    Girar imagem em um determinado ângulo
    :param imagem: Imagem
    :param angulo: Int -- em graus
    :return: Imagem
    '''
    return pg.transform.rotate(imagem, angulo)

'''
String, [Int, Int, Imagem] -> Imagem
Carrega imagem de arquivo. Se não for possível carregar, insere uma imagem substituta.
'''
def carregar_imagem(nome_arquivo, largura=100, altura=None, img_substituta=None):
    '''
    Carregar imagem pelo nome do arquivo. Pode-se definir as dimensões (largura e altura) e uma
    imagem substituta para o caso de não conseguir carregar do arquivo.
    Caso não seja fornecida imagem substituta e o arquivo não puder ser carregado, será desenhado
    um texto informando o problema.
    :param nome_arquivo: String
    :param largura: Int (opcional)
    :param altura: Int (opcional)
    :param img_substituta: Imagem (opcional)
    :return: Imagem
    '''
    try:
        img = pg.image.load(nome_arquivo)
        if largura and altura:
            img = definir_dimensoes(img, largura, altura)
        return img
    except:
        img = img_substituta if img_substituta \
                else texto("Não foi possível carregar imagem", Fonte("monospace", 15), Cor("red"), largura)
        return img

'''
Imagem, Imagem -> Imagem
Coloca uma imagem ao lado da outra
'''
def lado(img1, img2):
    '''
    Recebe duas imagens e as posiciona uma ao lado da outra.
    :param img1: Imagem
    :param img2: Imagem
    :return: Imagem
    '''
    fundo = folha_transparente(img1.get_width() + img2.get_width(),
                                    max(img1.get_height(), img2.get_height()))
    colocar_imagem(img1, fundo, img1.get_width()//2, fundo.get_height()//2)
    colocar_imagem(img2, fundo, img1.get_width() + img2.get_width()//2, fundo.get_height()//2)
    return fundo


'''
Imagem, Imagem -> Imagem
Coloca uma imagem acima da outra
'''
def encima(img1, img2):
    '''
    Recebe duas imagens e as posiciona uma acima (no plano 2D) da outra.
    :param img1: Imagem
    :param img2: Imagem
    :return: Imagem
    '''
    fundo = folha_transparente(max(img1.get_width(), img2.get_width()),
                               img1.get_height() + img2.get_height())
    colocar_imagem(img1, fundo, fundo.get_width()//2, img1.get_height()//2)
    colocar_imagem(img2, fundo, fundo.get_width()//2, img1.get_height() + img2.get_height()//2)
    return fundo

'''
Imagem, Imagem -> Imagem
Sobrepõe imagens, de modo a facilitar a geração de imagens personalizadas.
'''
def sobrepor(img1, img2):
    '''
    Recebe duas imagens e as sobrepõe (img1 encima de img2).
    :param img1: Imagem
    :param img2: Imagem
    :return: Imagem
    '''
    altura_maxima = max(img1.get_height(), img2.get_height())
    largura_maxima = max(img1.get_width(), img2.get_width())
    fundo = folha_transparente(largura_maxima, altura_maxima)
    fundo = colocar_imagem(img2, fundo, largura_maxima//2, altura_maxima//2)
    fundo = colocar_imagem(img1, fundo, largura_maxima//2, altura_maxima//2)
    return fundo


def largura_imagem(img):
    '''
    Retorna a largura da imagem
    :param img: Imagem
    :return: Int
    '''
    return img.get_width()

def altura_imagem(img):
    '''
    Retorna a altura da imagem
    :param img: Imagem
    :return: Int
    '''
    return img.get_height()


'''
FUNÇÕES DE CRIAÇÃO DE TELA E SOBREPOSIÇÕES
'''

'''
Surface, Surface, Int, Int -> Surface		
Coloca uma imagem (tipo pg.Surface) sobre outra na posição x e y, considerando 
a posição da imagem como seu centro.
'''
def colocar_imagem(img1, img2, x, y):
    '''
    Coloca img1 sobre a img2 na posição x e y.
    :param img1: Imagem
    :param img2: Imagem
    :param x: Int
    :param y: Int
    :return: Imagem
    '''
    img2.blit(img1, (x - img1.get_width()//2, y - img1.get_height()//2))
    return img2


def mostrar(funcao_desenha, *args):
    '''
    Recebe uma função que retorna uma imagem e mostra na tela.
    :param funcao_desenha: Function: QualquerCoisa -> Imagem -- função que retorna imagem
    :param args: parâmetros da 'funcao_desenha'
    :return: mostra imagem na tela
    '''
    while True:
        for event in pg.event.get():
            if event.type == pg.QUIT:
                sys.exit(0)
        pg.display.flip()
        funcao_desenha(*args)

'''
Surface, Int, Int -> void
'''
def colocar_imagem_sobre_tela_e_mostrar(img, x, y):
    '''
    Coloca a imagem sobre a tela principal na posição x e y e mostra na tela.
    :param img: Imagem
    :param x: Int
    :param y: Int
    :return: mostra imagem na tela
    '''
    mostrar(colocar_imagem, img, tela, x, y)


def mostrar_tela():
    '''
    Mostra tela principal com a imagem atual.
    :return: mostra imagem na tela
    '''
    while True:
        for event in pg.event.get():
            if event.type == pg.QUIT:
                sys.exit(0)
        pg.display.flip()
        # tela.blit(folha_transparente(0, 0), (0, 0))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from htdp_pt_br.image import *


def big_bang(inic,
             tela=tela,
             quando_tick=lambda e: e,
             frequencia=28,
             desenhar=lambda e: tela.blit(texto("NADA A MOSTRAR. VERIFIQUE SE VOCÊ PASSOU A FUNÇÃO DE DESENHHAR PARA O BIG-BANG", Fonte("monospace",30),
                                                Cor("red"), tela.get_width()), (0, tela.get_height()//2)),
             quando_tecla=lambda e, k: e, \
             quando_solta_tecla=lambda e, k: e, \
             quando_mouse=lambda e, x, y, ev: e, \
             parar_quando=lambda e: False,\
             modo_debug=False,
             fonte_debug = 15):
    '''
    Função que funciona como um framework para desenvolvimento de programas interativos.
    :param inic: EstadoMundo -- estado inicial do mundo (programa). O tipo EstadoMundo deve ser definido pelo programador.
    :param tela: Surface -- (opcional) insere tela diferente da padrão.
    :param quando_tick: Function: EstadoMundo -> EstadoMundo -- função que recebe o estado atual do mundo, gerando um novo estado.
    :param frequencia: Int -- frequência (frame rate) com que a função quando_tick é chamada.
    :param desenhar: Function: EstadoMundo -> Imagem -- função que recebe estado atual do mundo e cria uma imagem.
    :param quando_tecla: Function: EstadoMundo Tecla -> EstadoMundo -- função que recebe estado atual do mundo e o código de uma tecla (definido por pg.<NOME_TECLA>) quando a tecla é pressionada.
    :param quando_solta_tecla: Function: EstadoMundo Tecla -> EstadoMundo -- função que recebe estado atual do mundo e o código de uma tecla (definido por pg.<NOME_TECLA>) quando a tecla é solta.
    :param quando_mouse: Function: EstadoMundo EventoMouse Int Int -> EstadoMundo -- função que recebe estado atual do mundo, um código de evento de mouse (ex: pg.MOUSEBUTTONDOWN, pg.MOUSEBUTTONUP, pg.MOUSEMOTION), e a posição x e y em que o mouse se encontra na tela.
    :param parar_quando: Function: EstadoMundo -> Boolean -- função que recebe estado atual do mundo e responde se deve (True) ou não deve (False) parar o programa.
    :param modo_debug: Boolean -- flag que indica se devem aparecer informações de debug (escreve estado na tela).
    :param fonte_debug: Int -- tamanho da fonte do modo debug.
    :return: EstadoMundo -- estado final do mundo no momento em que programa foi parado.
    '''

    def verifica_valor_none(prox_estado, nome_funcao):
        if prox_estado is None:
            raise ValueError("""ERRO: Opa, parece que sua função '"""+nome_funcao+"""' retornou None. 
                    Verifique se você deu return no estado de seu programa!""")
        else:
            return prox_estado

    def quando_erro_invalido(err):
        raise ValueError("ERRO: Dados Inválidos\n\t--- "+str(err))
        sys.exit(0)

    estado = inic
    clock = pg.time.Clock()
    import time
    inic_time = time.time()


    while True:

        pg.display.flip()

        if parar_quando(estado):
            print(estado)
            end_time = time.time()
            print(end_time - inic_time)
            sys.exit(0)

        for event in pg.event.get():
            if event.type == pg.QUIT:
                print(estado)
                sys.exit(0)

            if event.type == pg.KEYDOWN:
                try:
                    prox_estado = quando_tecla(estado, event.key)
                    estado = verifica_valor_none(prox_estado, 'quando_tecla')
                except ValueError as err:
                    quando_erro_invalido(err)
            elif event.type == pg.KEYUP:
                try:
                    prox_estado = quando_solta_tecla(estado, event.key)
                    estado = verifica_valor_none(prox_estado, 'quando_solta_tecla')
                except ValueError as err:
                    quando_erro_invalido(err)
            elif event.type in [pg.MOUSEBUTTONDOWN, pg.MOUSEBUTTONUP, pg.MOUSEMOTION]:
                try:
                    x, y = pg.mouse.get_pos()
                    prox_estado = quando_mouse(estado, x, y, event.type)
                    estado = verifica_valor_none(prox_estado, 'quando_mouse')
                except ValueError as err:
                    quando_erro_invalido(err)
        try:
            prox_estado = quando_tick(estado)
            estado = verifica_valor_none(prox_estado, 'quando_tick')
        except ValueError as err:
            quando_erro_invalido(err)

        tela.fill(cor_fundo)
        desenhar(estado)
        if modo_debug:
            escreve_estado(estado, fonte_debug)

        clock.tick(frequencia)


def animar(a_cada_tick, frequencia=28, modo_debug=False, fonte_debug=15):
    '''
    Função que recebe uma função (a_cada_tick) que recebe um inteiro e cria uma imagem. A função 'a_cada_tick' é chamada
    repetidamente (de acordo com o parâmetro frequencia), sendo que em cada loop o valor passado por parâmetro é incrementado,
    possibilitando a criação de um filme (animação) com base na função 'a_cada_tick'.
    :param a_cada_tick: Function: Int -> Imagem -- função que recebe um número inteiro e gera, a partir disso, uma imagem
    :param frequencia: Int -- frequência (frame rate) com que a função 'a_cada_tick' é chamada.
    :param modo_debug: Boolean -- flag que indica se devem aparecer informações de debug (escreve estado na tela).
    :param fonte_debug: Int -- tamanho da fonte do modo debug.
    :return: Int -- estado final quando animação é fechada.
    '''
    clock = pg.time.Clock()
    i = 0
    while True:
        pg.display.flip()

        for event in pg.event.get():
            if event.type == pg.QUIT:
                print(i)
                sys.exit(0)

        tela.fill(COR_BRANCO)
        a_cada_tick(i)
        i += 1

        if modo_debug:
            escreve_estado(i, fonte_debug)

        clock.tick(frequencia)


def escreve_estado(estado, fonte_debug):
    '''
    Escreve na tela o estado interno.
    :param estado: EstadoMundo
    :param fonte_debug: Int -- tamanho da fonte do modo debug.
    :return: escreve o estado na tela.
    '''
    myfont = pg.font.SysFont("monospace", fonte_debug)
    # texto = str(estado).split(',')
    import re
    texto = re.findall('\[[^\]]*\]|\([^\)]*\)|\"[^\"]*\"|\S+', str(estado))

    counter = fonte_debug
    for line in texto:
        label = myfont.render(line, 1, (255, 0, 0))
        tela.blit(label, (5, counter))
        counter += fonte_debug





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="htdp_pt_br",
    version="0.0.8",
    author="Helio H. Monte-Alto",
    author_email="heliohenrique3@gmail.com",
    description="A library and framework based on the HtDP approach for teaching programming, by Matthias Felleisen et al. It is similar to the htdp_pt_br packages in Racket.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/helioh2/htdp-python-pt-br.git",
    packages=setuptools.find_packages(),
    install_requires=[
        'pygame', 'namedlist'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)